import asyncio
import logging

from .http_client import HttpClient
from .http_client import HttpException
from .bearer_token_http_client_provider import BearerTokenHttpClientProvider
from ._version import VERSION
import asyncio

_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

class RobustHttpClientProvider:
    """
    A HttpClient provider to make HTTP connections with protection against transient failures.
    """
    def __init__(self, http_client_provider, retries, backoff, product_name = None):
        """
        Initialize RobustHttpClientProvider class

        :param http_client_provider: A HttpClient provider for APIs using Bearer token authorization
        :param retries: The number of retries to attempt a failed request
        :param backoff: The amount of time to back off before a request retry
        :param product_name: The produce name of the consuming application
        """
        self._httpClientProvider = http_client_provider
        self._retries = retries
        self._backoff = backoff
        self._client = None
        self._version = VERSION

    async def retrieve_client(self):
        """
        Retrieves robust http client that helps to protect against transient failures.
        """
        if not self._client:
            self._client = self._RobustHttpClient(
                self._httpClientProvider,
                self._retries,
                self._backoff,
            )

        return self._client

    class _RobustHttpClient:

        # retry for the following status codes
        # 408 Request timeout
        # 409 Conflict
        # 418 I'm a teapot
        # 429 Too many requests
        # 500 Internal server error
        # 502 Bad gateway
        # 503 Service unavailable
        # 504 Gateway timeout
        SUPPORTED_EXCEPTIONS = [408, 409, 418, 429, 500, 502, 503, 504]

        def __init__(self, httpClientProvider: BearerTokenHttpClientProvider, retries=3, backoff=5):
            self._httpClientProvider = httpClientProvider
            self._retries = retries
            self._backoff = backoff

        async def post(self, url, value=None, headers=None):
            """ Http post request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.post(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "POST %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "POST %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    _logger.warning(
                        "POST try %d of %d to %s failed with retryable exception",
                        retries,
                        self._retries,
                        url,
                        exc_info=ex,
                    )
                    await asyncio.sleep(self._backoff)

            _logger.error(
                "POST %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def get(self, url, headers=None):
            """ Http get request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.get(url, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "GET %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "GET %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "GET %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def put(self, url, value=None, headers=None):
            """ Http put request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.put(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "PUT %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "PUT %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "PUT %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def patch(self, url, value=None, headers=None):
            """ Http patch request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.patch(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "PATCH %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "PATCH %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "PATCH %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def delete(self, url, headers=None):
            """ delete Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.delete(url, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "DELETE %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "DELETE %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "DELETE %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def post_json(self, url, value=None, headers=None):
            """ postJSON Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.post_json(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "POST json %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "POST json %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "POST json %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def get_json(self, url, headers=None):
            """ getJSON Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.get_json(url, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "GET json %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "GET json %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "GET json %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def put_json(self, url, value=None, headers=None):
            """ put_json Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.put_json(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "PUT json %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "PUT json %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "PUT json %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException

        async def patch_json(self, url, value=None, headers=None):
            """ patch_json Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.patch_json(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "PATCH json %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "PATCH json %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "PATCH json %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException
        
        async def post_full_response(self, url, value=None, headers=None):
            """ post full response Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.post_full_response(url, value, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "POST full response %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "POST full response %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "POST full response %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException
        
        async def delete_full_response(self, url, headers=None):
            """ delete full response Http request """
            headers = headers if headers is not None else {}
            intitialException = None
            retries = -1
            for retries in range(self._retries):
                try:
                    _httpClient = await self._httpClientProvider.retrieve_client()
                    return await _httpClient.delete_full_response(url, headers)
                except HttpException as ex:
                    if ex.status not in self.SUPPORTED_EXCEPTIONS:
                        _logger.error(
                            "DELETE full response %s try %d raised an unsupported exception: %s",
                            url,
                            retries,
                            str(ex),
                            exc_info=ex,
                        )
                        raise
                    else:
                        _logger.info(
                            "DELETE full response %s try %d raised a supported exception %s",
                            url,
                            retries,
                            str(ex),
                        )

                    if not intitialException:
                        intitialException = ex

                    await asyncio.sleep(self._backoff)

            _logger.error(
                "DELETE full response %s giving up after try %d. Initial exception was %s",
                url,
                retries,
                str(intitialException),
            )
            raise intitialException


class HttpException(Exception):
    def __init__(self, status, message):
        self.status = status
        super().__init__(message)

    def __str__(self):
        return 'Returned: ' + str(self.status) + ' ' + super().__str__()


import json

from .http_client import HttpClient
from ._version import VERSION

class RefreshableTokenProvider:
    """
    A refreshable token provider based on the OAuth refresh token grant type
    """
    def __init__(self, endpoint_provider, client_id, client_secret, token, product_name = None):
        """
        Initialize RefreshableTokenProvider class

        :param endpoint_provider: An endpoint provider that provides the URL for the Trimble Identity token endpoint
        :param client_id: The consumer key for the calling application
        :param client_secret: The consumer secret for the calling application
        :param token: The initial access token issued for the authenticated user
        :param product_name: Product name of the calling application
        """
        self._endpointProvider = endpoint_provider
        self._consumerKey = client_id
        self._consumerSecret = client_secret
        self._token = token

        self._version = VERSION
        self._productName = product_name

    async def retrieve_token(self):
        """
        Retrieves an access token for the application using refresh grant type

        :return: The access token for the given application
        """
        tokenEndpoint = await self._endpointProvider.retrieve_token_endpoint()
        client = HttpClient('', {})
        result = await client.post(
            tokenEndpoint, 
            'grant_type=refresh_token&client_id=' + self._consumerKey + '&client_secret=' + self._consumerSecret, 
            { 'content-type': 'application/x-www-form-urlencoded', 'accept': 'application/json', 'user-agent': f"TrimbleCloud.Authentication/{self._version} python-sdk {self._productName}" }
        )
        jsonResult = json.loads(result)
        return jsonResult['access_token']


from .http_client import HttpClient
from ._version import VERSION 

class OpenIdEndpointProvider:
    """
    An endpoint provider that returns values from a OpenID well known configuration
    """
    def __init__(self, configuration_url, product_name = None):
        """
        Initialize OpenIdEndpointProvider class

        :param configurationUrl: The URL for the Trimble Identity OpenID well know configuration endpoint
        :param productName: Product name of consuming application
        """
        self._configurationUrl = configuration_url
        self._configuration = None

        self._version = VERSION
    
    async def retrieve_authorization_endpoint(self):
        """
        Retrieves authorization endpoint from a OpenID well known configuration
        """
        if self._configuration is None:
            await self._load_configuration()
        return self._configuration['authorization_endpoint']

    async def retrieve_token_endpoint(self):
        """
        Retrieves token endpoint from a OpenID well known configuration
        """
        if self._configuration is None:
            await self._load_configuration()
        return self._configuration['token_endpoint']

    async def retrieve_jwks_endpoint(self):
        """
        Retrieves JWKS endpoint from a OpenID well known configuration
        """
        if self._configuration is None:
            await self._load_configuration()
        return self._configuration['jwks_uri']

    async def _load_configuration(self):
        client = HttpClient('', {})
        self._configuration = await client.get_json(self._configurationUrl, {})


from ._version import VERSION

class FixedEndpointProvider:
    """
    An endpoint provider that returns fixed OAuth endpoints
    """
    def __init__(
            self, 
            authorization_endpoint, 
            token_endpoint, 
            userinfo_endpoint, 
            token_revocation_endpoint = None, 
            jwks_endpoint = None, 
            product_name = None):
        """
        Initialize FixedEndpointProvider class

        :param authorization_endpoint: Set Authorization Endpoint
        :param token_endpoint: Set Token Endpoint
        :param userinfo_endpoint: Set UserInfo Endpoint
        :param token_revocation_endpoint: Set Token revocation Endpoint
        :param jwks_endpoint: Set JSON Web key set Endpoint
        :param product_name: Product name of the consuming application (optional)
        """
        self._authorizationEndpoint = authorization_endpoint
        self._tokenEndpoint = token_endpoint
        self._userInfoEndpoint = userinfo_endpoint
        self._tokenRevocationEndpoint = token_revocation_endpoint
        self._jwksEndpoint = jwks_endpoint
        self._version = VERSION

    async def retrieve_authorization_endpoint(self):
        """
        Retrieves a fixed Authorization endpoint
        """
        return self._authorizationEndpoint

    async def retrieve_token_endpoint(self):
        """
        Retrieves a fixed Token endpoint
        """
        return self._tokenEndpoint

    async def retrieve_json_web_keyset_endpoint(self):
        """
        Retrieves a fixed JSON Web key set endpoint
        """
        return self._jwksEndpoint


from ._version import VERSION

class FixedKeySetProvider:
    """
    A keyset provider that returns a fixed keyset
    """
    def __init__(self, keyset, product_name = None):      
        """
        Initialize FixedKeySetProvider class

        :param keyset: A dictionary of named keys
        :param product_name: Product name of consuming application
        """  
        self._keyset = keyset

        self._version = VERSION

    async def retrieve_keyset(self):
        """
        Retrieves a dictionary of named keys

        :return: Fixed Keyset
        """
        return self._keyset


__path__ = __import__('pkgutil').extend_path(__path__, __name__)

from .bearer_token_http_client_provider import BearerTokenHttpClientProvider
from .client_credential_token_provider import ClientCredentialTokenProvider
from .fixed_endpoint_provider import FixedEndpointProvider
from .fixed_keyset_provider import FixedKeySetProvider
from .fixed_token_provider import FixedTokenProvider
from .open_id_endpoint_provider import OpenIdEndpointProvider
from .open_id_keyset_provider import OpenIdKeySetProvider
from .refreshable_token_provider import RefreshableTokenProvider
from .robust_http_client_provider import RobustHttpClientProvider
from .validated_claimset_provider import ValidatedClaimsetProvider

__all__ = [
    'BearerTokenHttpClientProvider',
    'ClientCredentialTokenProvider',
    'FixedEndpointProvider',
    'FixedKeySetProvider',
    'FixedTokenProvider',
    'OpenIdEndpointProvider',
    'OpenIdKeySetProvider',
    'RefreshableTokenProvider',
    'RobustHttpClientProvider',
    'ValidatedClaimsetProvider'
]


import base64
from datetime import datetime, timedelta
import json
import urllib

from trimble.id._constants import PACKAGE_NAME
from .http_client import HttpClient
from ._version import VERSION

class ClientCredentialTokenProvider:
    """
    A token provider based on the OAuth Client Credential grant type
    """
    def __init__(self, endpoint_provider, client_id, client_secret, product_name=None):
        """
        Initialize ClientCredentialTokenProvider class

        :param endpoint_provider: An endpoint provider that provides the URL for the Trimble Identity token endpoint
        :param client_id: The consumer key for the calling application
        :param client_secret: The consumer secret of the calling application
        :param product_name: Product name of consuming application (optional)
        """
        self._endpointProvider = endpoint_provider
        self._consumerKey = client_id
        self._consumerSecret = client_secret
        self._scopes = None
        self._tokenExpiry = datetime.min
        self._accessToken = None
        self._version = VERSION
        self._productName = product_name

    def with_scopes(self, scopes):
        """
        A method for adding scopes

        :params scopes: The requested scopes of calling application
        """
        self._scopes = scopes
        return self

    async def retrieve_token(self):
        """
        Retrieves an access token for the application

        :return: access token for the given application
        """       
        if self._tokenExpiry < datetime.utcnow():
            await self._refresh_token()
        return self._accessToken

    async def _refresh_token(self):
        tokenEndpoint = await self._endpointProvider.retrieve_token_endpoint()
        client = HttpClient('', {})
        basicAuthorizationValue = base64.b64encode(f'{self._consumerKey}:{self._consumerSecret}'.encode('ascii')).decode('ascii')
        parameters = {
            'grant_type': 'client_credentials'
        }
        if self._scopes is not None:
            parameters['scope'] = ' '.join(self._scopes)
        result = await client.post(
            tokenEndpoint,
            urllib.parse.urlencode(parameters),
            {
                'authorization': f'Basic {basicAuthorizationValue}',
                'content-type': 'application/x-www-form-urlencoded',
                'accept': 'application/json',
                'user-agent': f"{PACKAGE_NAME}/{self._version} python-sdk {self._productName}"
            }
        )
        jsonResult = json.loads(result)
        self._tokenExpiry = datetime.utcnow() + timedelta(0, jsonResult['expires_in'])
        self._accessToken = jsonResult['access_token']



import json
import jwt

from .http_client import HttpClient
from ._version import VERSION

class OpenIdKeySetProvider:
    """
    A keyset provider based on the OAuth well known configuration
    """
    def __init__(self, endpoint_provider, product_name = None):
        """
        Initialize OpenIdKeySetProvider class

        :param endpoint_provider: An endpoint provider that provides the URL for the Trimble Identity JSON web keyset endpoint
        :param product_name: Product name of the consuming application
        """
        self._endpointProvider = endpoint_provider

        self._version = VERSION

    async def retrieve_keys(self):
        """
        Retrieves an dictionary of named keys
        """
        client = HttpClient('', {})
        result = await client.get_json(await self._endpointProvider.retrieve_jwks_endpoint(), {})
        self._keys = {}
        for jwk in result['keys']:
            kid = jwk['kid']
            self._keys[kid] = jwt.algorithms.RSAAlgorithm.from_jwk(json.dumps(jwk))
        return self._keys


from .http_client import HttpClient
from ._version import VERSION

class BearerTokenHttpClientProvider:
    """
    A HttpClient provider for APIs using Bearer token authorization
    """
    def __init__(self, token_provider, base_address, default_headers = {}, product_name=None):
        """
        Initialize Bearer token http client provider

        :param token_provider: A token provider that provides the access token for the authenticated application or user
        :param base_address: The base address for the API that will be called
        :param default_headers: If any default address to be applied in Http request call. (Optional)
        :param product_name: Specify product name of consuming application (optional)
        """
        self._tokenProvider = token_provider
        self._baseAddress = base_address
        self._defaultHeaders = default_headers
        self._version = VERSION
        self._consumerKey = self.retrieve_consumer_key()

    async def retrieve_client(self):
        """
        Retrieves a preconfigured HttpClient to access a given API

        :return: A preconfigured HttpClient to access any given API
        """
        token = await self._tokenProvider.retrieve_token()
        
        if 'trimblepaas.com' in self._baseAddress:
            url = self._baseAddress + 'api/'
        else:
            url = self._baseAddress

        url = self._add_trailing_slash(url)

        return HttpClient(url, { **self._defaultHeaders, **{ 'authorization': 'Bearer ' + token } })

    def _add_trailing_slash(self, url):
        if url.endswith('/'):
            return url
        return url + '/'
    
    def retrieve_consumer_key(self):
        if (self._tokenProvider != None) : 
            return self._tokenProvider._consumerKey
        return None
    



from ._version import VERSION

class FixedTokenProvider:
    """
    A token provider that returns a fixed token
    """
    def __init__(self, token, product_name = None):
        """
        Initialize FixedTokenProvider class

        :param token: Sets access token to FixedTokenProvider
        :param productName: Product name of the consuming application
        """
        self._token = token
        self._consumerKey = None
        self._version = VERSION
    
    def with_token(self, token):
        """
        Sets consumer key to FixedTokenProvider

        :param consumer_key: Sets consumer key to FixedTokenProvider
        """
        self._token = token
        return self

    async def retrieve_token(self):
        """
        Retrieves fixed token
        """
        return self._token
    


import jwt
from ._version import VERSION

class ValidatedClaimsetProvider:
    """
    A claimset provider that returns a validated claimset
    """
    def __init__(self, jwk_provider, product_name = None):
        """
        Initialize ValidatedClaimsetProvider class

        :param jwk_provider: A provider for the keyset used to validate the JWT claimeset
        :param product_name: The product name of the consuming application
        """
        self._jwksProvider = jwk_provider
        self._keys = None
        self._version = VERSION

    async def retrieve_claimset(self, token):
        """
        Retrieves a validate claimset from a given JSON web token
        """
        if self._keys == None:
            self._keys = await self._jwksProvider.retrieve_keys()        
        kid = jwt.get_unverified_header(token)['kid']
        if not kid in self._keys:
            self._keys = await self._jwksProvider.retrieve_keys()
            if not kid in self._keys:
                raise Exception('No matching key in JWKS')
        
        # It is important that you hardcode the algorithm and not use the value supplied in the JWT header
        # Don't validate the audience claim. Need to relook this necessity
        return jwt.decode(token, key=self._keys[kid], algorithms=['RS256'], options={'verify_aud': False}) 



# setup.py
import re
import os.path
from setuptools import setup

PACKAGE_NAME = "trimble-id"
PACKAGE_PPRINT_NAME = "Authentication module"

package_folder_path = PACKAGE_NAME.replace('-', '/')

with open(os.path.join(package_folder_path, 'version.py')
          if os.path.exists(os.path.join(package_folder_path, 'version.py'))
          else os.path.join(package_folder_path, '_version.py'), 'r') as fd:
    version = re.search(r'^VERSION\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open("README.md", encoding="utf-8") as f:
    readme = f.read()

setup(name=PACKAGE_NAME,
      version=version,
      description='Trimble {} Client Library for Python'.format(PACKAGE_PPRINT_NAME),
      url='https://github.com/trimble-oss/trimble-id-sdk-docs-for-python',
      long_description=readme,
      long_description_content_type="text/markdown",
      license="MIT License",
      author="Trimble Inc.",
      author_email="sdk@trimble.com",
      keywords="Trimble, TrimbleID, Trimble Identity",
      classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
      ],
      scripts=[],
      packages=['trimble.id'],
      setup_requires=[],
      install_requires=[
            'aiohttp>=3.7.4',
            'PyJWT>=2.4.0',
            "requests>=2.28.1",
            "cryptography>=38.0.3"
            ],
      tests_require=[],
      extras_require=dict()
)



import aiohttp
import json
from urllib.parse import urljoin

from .http_exception import HttpException


class HttpClient:
    def __init__(self, base_address, default_headers):
        self._baseAddress = base_address
        self._defaultHeaders = default_headers

    async def post(self, url, value=None, headers={}):
        async with aiohttp.ClientSession() as session:
            async with session.post(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers),
                data=value
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())

                return await response.text()

    async def get(self, url, headers={}):
        async with aiohttp.ClientSession() as session:
            async with session.get(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers)
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())

                return await response.text()

    async def put(self, url, value=None, headers={}):
        async with aiohttp.ClientSession() as session:
            async with session.put(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers),
                data=value
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())

                return await response.text()

    async def patch(self, url, value=None, headers={}):
        async with aiohttp.ClientSession() as session:
            async with session.patch(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers),
                data=value
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())

                return await response.text()

    async def delete(self, url, headers={}):
        async with aiohttp.ClientSession() as session:
            async with session.delete(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers)
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())

                return await response.text()

    async def post_json(self, url, value=None, headers={}):
        result = await self.post(
            url,
            None if value is None else json.dumps(value),
            self._combine_headers(
                {"accept": "application/json", "content-type": "application/json"},
                headers
            )
        )
        return json.loads(result)

    async def get_json(self, url, headers={}):
        result = await self.get(
            url, self._combine_headers({"accept": "application/json"}, headers)
        )
        return json.loads(result)

    async def put_json(self, url, value=None, headers={}):
        result = await self.put(
            url,
            None if value is None else json.dumps(value),
            self._combine_headers(
                {"accept": "application/json", "content-type": "application/json"},
                headers
            )
        )
        return json.loads(result)

    async def patch_json(self, url, value=None, headers={}):
        result = await self.patch(
            url,
            None if value is None else json.dumps(value),
            self._combine_headers(
                {"accept": "application/json", "content-type": "application/json"},
                headers
            )
        )
        return json.loads(result)
    
    async def post_full_response(self, url, value=None, headers={}):
        async with aiohttp.ClientSession() as session:
            if value is not None:
                value = json.dumps(value)
                headers = self._combine_headers(
                    {"accept": "application/json", "content-type": "application/json"},
                    headers
                )
            async with session.post(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers),
                data=value
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())
                
                response_text = await response.text()
                return (json.loads(response_text), response)
            
    async def delete_full_response(self, url, headers={}):
        async with aiohttp.ClientSession() as session:
            async with session.delete(
                self._combine_url(self._baseAddress, url),
                headers=self._combine_headers(self._defaultHeaders, headers)
            ) as response:
                if response.status >= 400:
                    raise HttpException(response.status, await response.text())
                
                return response

    def _combine_url(self, baseAddress, url):
        return urljoin(baseAddress, url)

    def _combine_headers(self, baseHeaders, headers):
        return {**baseHeaders, **headers}
